/*
 * File: TMCourse.js
 * -----------------
 * This class defines the data structure for a course for use with
 * the TeachingMachine program.
 */

/*
 * Creates a TMCourse object by extracting data from 
 * the index.html file.
 */

class TMCourse {

    private currentQuestion: TMQuestion;
    private questions: QuestionMap;

    constructor() {
        let questions = readQuestions();
        if (questions === undefined) throw "Could not read questions!";
        this.currentQuestion = questions["START"];
        this.questions = questions;
    }

    askQuestion() {
        this.currentQuestion.printQuestionText();

        // .bind(this) is necessary if you're using a method as a callback.
        console.requestInput("> ", this.checkAnswer.bind(this));
    }

    checkAnswer(line: string): void {
        let questionID = this.currentQuestion.getNextQuestion(line);
        if (questionID === "EXIT") return;
        if (questionID === undefined) {
            console.log("I don't understand that response.");
        } else {
            this.currentQuestion = this.questions[questionID];
        }
        this.askQuestion();
    }

    run(): void {
        this.askQuestion()
    }
}
